@@ -3,6 +3,7 @@
 
 import base64
 import binascii
+import hashlib
 import os
 import json
 import logging
@@ -18,6 +19,7 @@ uri = 'http://music.163.com/api'
 uri_we = 'http://music.163.com/weapi'
 uri_v1 = 'http://music.163.com/weapi/v1'
 uri_v3 = 'http://music.163.com/weapi/v3'
+uri_e = 'https://music.163.com/eapi'
 
 logger = logging.getLogger(__name__)
 
@@ -50,6 +52,8 @@ class API(object):
 
     def load_cookies(self, cookies):
         self._cookies.update(cookies)
+        # 云盘资源发布仅有似乎不支持osx平台
+        self._cookies.update(dict(appver="7.2.24", os="android"))
 
     def set_http(self, http):
         self._http = http
@@ -397,12 +401,13 @@ class API(object):
             'op': op   # opation
         }
         data = self.request('POST', url_add, data_add)
+        print(data)
         code = data.get('code')
 
         # 从歌单中成功的移除歌曲时，code 是 200
         # 当从歌单中移除一首不存在的歌曲时，code 也是 200
-        # 当向歌单添加歌曲时，如果歌曲已经在列表当中，
-        # 返回 code 为 502
+        # 当向歌单添加歌曲时，如果歌曲已经在列表当中，返回 code 为 502
+        # code 为 521 时，可能是因为：绑定手机号后才可操作哦
         if code == 200:
             return 1
         elif code == 502:
@@ -433,9 +438,9 @@ class API(object):
         """
         url = uri + '/mv/detail?id=' + str(mvid)
         data = self.request('GET', url)
-        if data['code'] == 404:
-            return None
-        return data
+        if data['code'] == 200:
+            return data['data']
+        raise CodeShouldBe200(data)
 
     def get_lyric_by_songid(self, mid):
         """Get song lyric
@@ -505,6 +510,124 @@ class API(object):
         payload = self.encrypt_request(data)
         return self.request('POST', url, payload)
 
+    def cloud_songs(self, offset=0, limit=30):
+        data = dict(limit=limit, offset=offset)
+        url = uri_v1 + '/cloud/get'
+        payload = self.encrypt_request(data)
+        res_data = self.request('POST', url, payload)
+        if res_data['code'] == 200:
+            return res_data
+        raise CodeShouldBe200(res_data)
+
+    def cloud_songs_detail(self, music_ids):
+        data = dict(songIds=music_ids.split(","))
+        url = uri_v1 + '/cloud/get/byids'
+        payload = self.encrypt_request(data)
+        return self.request('POST', url, payload)
+
+    def cloud_songs_delete(self, music_ids):
+        data = dict(songIds=music_ids.split(","))
+        url = uri_we + '/cloud/del'
+        payload = self.encrypt_request(data)
+        return self.request('POST', url, payload)
+
+    def cloud_song_match(self, sid, asid):
+        url = uri + f'/cloud/user/song/match?songId={sid}&adjustSongId={asid}'
+        data = self.request('GET', url)
+        if data['code'] == 200:
+            return data['data']
+        raise CodeShouldBe200(data)
+
+    def cloud_song_upload(self, path):
+        def md5sum(file):
+            md5sum = hashlib.md5()
+            with open(file, 'rb') as f:
+                # while chunk := f.read():
+                #     md5sum.update(chunk)
+                md5sum.update(f.read())
+            return md5sum
+
+        from .cloud_helpers.cloud_api import Cloud_API
+        cloud_api = Cloud_API(self, uri_e)
+
+        fname = os.path.basename(path)
+        fext = path.split('.')[-1]
+        '''Parsing file names'''
+        fsize = os.stat(path).st_size
+        md5 = md5sum(path).hexdigest()
+        logger.debug(f'[-] Checking file ( MD5: {md5} )')
+        cresult = cloud_api.GetCheckCloudUpload(md5)
+        if cresult['code'] != 200:
+            return 'UPLOAD_CHECK_FAILED'
+
+        '''网盘资源发布 4 步走：'''
+        '''1.拿到上传令牌 - 需要文件名，MD5，文件大小'''
+        token = cloud_api.GetNosToken(fname, md5, fsize, fext)
+        if token['code'] != 200:
+            return 'TOKEN_ALLOC_FAILED'
+        token = token['result']
+
+        '''2. 若文件未曾上传完毕，则完成其上传'''
+        if cresult['needUpload']:
+            logger.info(f'[+] {fname} needs to be uploaded ( {fsize} B )')
+            try:
+                upload_result = cloud_api.SetUploadObject(
+                    open(path, 'rb'),
+                    md5, fsize, token['objectKey'], token['token']
+                )
+            except Exception:
+                return 'UPLOAD_FAILED'
+            logger.debug(f'[-] Response:\n  {upload_result}')
+
+        '''3. 提交资源'''
+        songId = cresult['songId']
+        logger.debug(f'''[!] Assuming upload has finished,preparing to submit
+        ID  :   {songId}
+        MD5 :   {md5}
+        NAME:   {fname}''')
+        metadata = cloud_api.GetMetadata(path)
+        submit_result = cloud_api.SetUploadCloudInfo(
+            token['resourceId'], songId, md5, fname,
+            song=metadata.get('title', '.'),
+            artist=metadata.get('artist', '.'),
+            album=metadata.get('album', '.')
+        )
+        if submit_result['code'] != 200:
+            return 'SUBMIT_FAILED'
+        logger.debug(f'[-] Response:\n  {submit_result}')
+
+        '''4. 发布资源'''
+        publish_result = cloud_api.SetPublishCloudResource(submit_result['songId'])
+        if publish_result['code'] != 200:
+            return 'PUBLISH_FAILED'
+        logger.debug(f'[-] Response:\n  {publish_result}')
+
+        return 'STATUS_SUCCEEDED'
+
+    def subscribed_djradio(self, limit=0, offset=0):
+        data = dict(limit=100, time=0, needFee=False)
+        url = uri_e + '/djradio/subed/v1'
+        payload = self.eapi_encrypt(b'/api/djradio/subed/v1', data)
+        return self.request('POST', url, {'params': payload})
+
+    def djradio_detail(self, radio_id):
+        data = dict(id=radio_id)
+        url = uri_e + '/djradio/v2/get'
+        payload = self.eapi_encrypt(b'/api/djradio/v2/get', data)
+        return self.request('POST', url, {'params': payload})
+
+    def djradio_song_detail(self, id_):
+        data = dict(id=id_)
+        url = uri_e + '/dj/program/detail'
+        payload = self.eapi_encrypt(b'/api/dj/program/detail', data)
+        return self.request('POST', url, {'params': payload})
+
+    def djradio_list(self, radio_id, limit=50, offset=0, asc=False):
+        data = dict(radioId=radio_id, limit=limit, offset=offset, asc=asc)
+        url = uri_e + '/v1/dj/program/byradio'
+        payload = self.eapi_encrypt(b'/api/v1/dj/program/byradio', data)
+        return self.request('POST', url, {'params': payload})
+
     def _create_aes_key(self, size):
         return (''.join([hex(b)[2:] for b in os.urandom(size)]))[0:16]
 
@@ -528,6 +651,25 @@ class API(object):
                              int(e, 16), int(n, 16))
         return format(encrypted_text, "x").zfill(256)
 
+    def eapi_encrypt(self, path, params):
+        """
+        eapi接口参数加密
+        :param bytes path: 请求的路径
+        :param params: 请求参数
+        :return str: 加密结果
+        """
+        params = json.dumps(params, separators=(',', ':')).encode()
+        sign_src = b'nobody' + path + b'use' + params + b'md5forencrypt'
+        m = hashlib.md5()
+        m.update(sign_src)
+        sign = m.hexdigest()
+        aes_src = path + b'-36cd479b6b5-' + params + b'-36cd479b6b5-' + sign.encode()
+        pad = 16 - len(aes_src) % 16
+        aes_src = aes_src + bytearray([pad] * pad)
+        crypt = AES.new(b'e82ckenh8dichen8', AES.MODE_ECB)
+        ret = crypt.encrypt(aes_src)
+        return binascii.b2a_hex(ret).upper()
+
     def encrypt_request(self, data):
         text = json.dumps(data)
         first_aes_key = '0CoJUm6Qyw8W8jud'
@@ -544,3 +686,10 @@ class API(object):
 
 
 api = API()
+
+
+if __name__ == '__main__':
+    from fuo_netease.login_controller import LoginController
+    user = LoginController.load()
+    api.load_cookies(user.cookies)
+    print(api.djradio_song_detail(1883706033))
